@@ -38,6 +38,7 @@ class SystemTestCase(base.TestCase):
     def setUp(self):
         super(SystemTestCase, self).setUp()
         self.conn = mock.Mock()
+        self.conn.get.return_value.headers = {'Allow': 'GET,HEAD'}
         with open('sushy/tests/unit/json_samples/system.json') as f:
             self.json_doc = json.load(f)
 
@@ -254,6 +255,7 @@ class SystemTestCase(base.TestCase):
                         sushy.BOOT_SOURCE_TARGET_PXE,
                         sushy.BOOT_SOURCE_TARGET_CD,
                         sushy.BOOT_SOURCE_TARGET_USB,
+                        sushy.BOOT_SOURCE_TARGET_USB_CD,
                         sushy.BOOT_SOURCE_TARGET_HDD,
                         sushy.BOOT_SOURCE_TARGET_BIOS_SETUP,
                         sushy.BOOT_SOURCE_TARGET_UTILITIES,
@@ -276,7 +278,8 @@ class SystemTestCase(base.TestCase):
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
                            'BootSourceOverrideTarget': 'Pxe',
-                           'BootSourceOverrideMode': 'UEFI'}})
+                           'BootSourceOverrideMode': 'UEFI'}},
+            headers=None)
 
     def test_set_system_boot_options_no_mode_specified(self):
         self.sys_inst.set_system_boot_options(
@@ -285,7 +288,8 @@ class SystemTestCase(base.TestCase):
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Once',
-                           'BootSourceOverrideTarget': 'Hdd'}})
+                           'BootSourceOverrideTarget': 'Hdd'}},
+            headers=None)
 
     def test_set_system_boot_options_no_target_specified(self):
         self.sys_inst.set_system_boot_options(
@@ -294,7 +298,8 @@ class SystemTestCase(base.TestCase):
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
-                           'BootSourceOverrideMode': 'UEFI'}})
+                           'BootSourceOverrideMode': 'UEFI'}},
+            headers=None)
 
     def test_set_system_boot_options_no_freq_specified(self):
         self.sys_inst.set_system_boot_options(
@@ -303,12 +308,14 @@ class SystemTestCase(base.TestCase):
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideTarget': 'Pxe',
-                           'BootSourceOverrideMode': 'UEFI'}})
+                           'BootSourceOverrideMode': 'UEFI'}},
+            headers=None)
 
     def test_set_system_boot_options_nothing_specified(self):
         self.sys_inst.set_system_boot_options()
         self.sys_inst._conn.patch.assert_called_once_with(
-            '/redfish/v1/Systems/437XR1138R2', data={})
+            '/redfish/v1/Systems/437XR1138R2', data={},
+            headers=None)
 
     def test_set_system_boot_options_invalid_target(self):
         self.assertRaises(exceptions.InvalidParameterValueError,
@@ -325,6 +332,71 @@ class SystemTestCase(base.TestCase):
                 sushy.BOOT_SOURCE_TARGET_HDD,
                 enabled='invalid-enabled')
 
+    def test_set_system_boot_options_supermicro_usb_cd_boot(self):
+        (self.json_doc["Boot"]
+         ["BootSourceOverrideTarget@Redfish.AllowableValues"]).append("UsbCd")
+        self.sys_inst._parse_attributes(self.json_doc)
+
+        self.sys_inst.manufacturer = "supermicro"
+        self.sys_inst.set_system_boot_options(
+            target=sushy.BOOT_SOURCE_TARGET_CD,
+            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
+
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2',
+            data={'Boot': {'BootSourceOverrideEnabled': 'Once',
+                           'BootSourceOverrideTarget': 'UsbCd'}},
+            headers=None)
+
+    def test_set_system_boot_options_supermicro_no_usb_cd_boot(self):
+
+        self.sys_inst.manufacturer = "supermicro"
+        self.sys_inst.set_system_boot_options(
+            target=sushy.BOOT_SOURCE_TARGET_CD,
+            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
+
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2',
+            data={'Boot': {'BootSourceOverrideEnabled': 'Once',
+                           'BootSourceOverrideTarget': 'Cd'}},
+            headers=None)
+
+    def test_set_system_boot_options_settings_resource_nokia(self):
+        self.conn.get.return_value.headers = {'ETag': '"3d7b838291941d"'}
+        with open('sushy/tests/unit/json_samples/settings-nokia.json') as f:
+            settings_obj = json.load(f)
+
+        self.json_doc.update(settings_obj)
+        self.sys_inst._parse_attributes(self.json_doc)
+
+        self.sys_inst.set_system_boot_options(
+            target=sushy.BOOT_SOURCE_TARGET_CD,
+            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
+
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/Self/SD',
+            data={'Boot': {'BootSourceOverrideEnabled': 'Once',
+                           'BootSourceOverrideTarget': 'Cd'}},
+            headers={'If-Match': '"3d7b838291941d"'})
+
+    def test_set_system_boot_options_settings_resource(self):
+        self.conn.get.return_value.headers = {'ETag': '"3d7b838291941d"'}
+        with open('sushy/tests/unit/json_samples/settings.json') as f:
+            settings_obj = json.load(f)
+
+        self.json_doc.update(settings_obj)
+        self.sys_inst._parse_attributes(self.json_doc)
+
+        self.sys_inst.set_system_boot_options(
+            target=sushy.BOOT_SOURCE_TARGET_CD,
+            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
+
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2/BIOS/Settings',
+            data={'Boot': {'BootSourceOverrideEnabled': 'Once',
+                           'BootSourceOverrideTarget': 'Cd'}},
+            headers={'If-Match': '"3d7b838291941d"'})
+
     def test_set_system_boot_source(self):
         self.sys_inst.set_system_boot_source(
             sushy.BOOT_SOURCE_TARGET_PXE,
@@ -334,7 +406,21 @@ class SystemTestCase(base.TestCase):
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
                            'BootSourceOverrideTarget': 'Pxe',
-                           'BootSourceOverrideMode': 'UEFI'}})
+                           'BootSourceOverrideMode': 'UEFI'}},
+            headers=None)
+
+    def test_set_system_boot_source_with_etag(self):
+        self.conn.get.return_value.headers = {'ETag': '"3d7b838291941d"'}
+        self.sys_inst.set_system_boot_source(
+            sushy.BOOT_SOURCE_TARGET_PXE,
+            enabled=sushy.BOOT_SOURCE_ENABLED_CONTINUOUS,
+            mode=sushy.BOOT_SOURCE_MODE_UEFI)
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2',
+            data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
+                           'BootSourceOverrideTarget': 'Pxe',
+                           'BootSourceOverrideMode': 'UEFI'}},
+            headers={'If-Match': '"3d7b838291941d"'})
 
     def test_set_system_boot_source_no_mode_specified(self):
         self.sys_inst.set_system_boot_source(
@@ -343,7 +429,8 @@ class SystemTestCase(base.TestCase):
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Once',
-                           'BootSourceOverrideTarget': 'Hdd'}})
+                           'BootSourceOverrideTarget': 'Hdd'}},
+            headers=None)
 
     def test_set_system_boot_source_invalid_target(self):
         self.assertRaises(exceptions.InvalidParameterValueError,
